@@ -1,6 +1,7 @@
 """Sensor classes for SMA WebConnect library for Python."""
 import copy
 import logging
+from typing import Any, Iterator, List, Optional, Union
 
 import attr
 import jmespath  # type: ignore
@@ -11,42 +12,42 @@ _LOGGER = logging.getLogger(__name__)
 
 
 @attr.s(slots=True)
-class Sensor:  # pylint: disable=too-many-instance-attributes
+class Sensor:
     """pysma sensor."""
 
-    key = attr.ib()
-    name = attr.ib()
-    unit = attr.ib(default="")
-    factor = attr.ib(default=None)
-    path = attr.ib(default=None)
-    enabled = attr.ib(default=True)
-    l10n_translate = attr.ib(default=False)
-    value = attr.ib(default=None, init=False)
-    key_idx = attr.ib(default="0", repr=False, init=False)
+    # pylint: disable=too-many-instance-attributes
+    # pylint: disable=too-few-public-methods
+    key: str = attr.ib()
+    name: str = attr.ib()
+    unit: str = attr.ib(default="")
+    factor: int = attr.ib(default=None)
+    path: Union[list, tuple] = attr.ib(default=None)
+    enabled: bool = attr.ib(default=True)
+    l10n_translate: bool = attr.ib(default=False)
+    value: Any = attr.ib(default=None, init=False)
+    key_idx: int = attr.ib(default=0, repr=False, init=False)
 
-    def __attrs_post_init__(self):
-        """Init path."""
+    def __attrs_post_init__(self) -> None:
+        """Post init Sensor."""
         key = str(self.key)
         skey = key.split("_")
         if len(skey) > 2 and skey[2].isdigit():
             self.key = f"{skey[0]}_{skey[1]}"
-            self.key_idx = skey[2]
+            self.key_idx = int(skey[2])
 
-    def extract_logger(self, result_body):
-        """Extract logs from json body.
-
-        Args:
-            result_body: json body retrieved from device
-        """
-        self.value = result_body
-
-    def extract_value(self, result_body, l10n=None, devclass="1"):
+    def extract_value(
+        self, result_body: dict, l10n: Optional[dict] = None, devclass: str = "1"
+    ) -> bool:
         """Extract value from json body.
 
         Args:
-            result_body: json body retrieved from device
-            l10n: Dictionary to translate tags to strings
-            devclass: The device class of the device used to extract the value
+            result_body (dict): json body retrieved from device
+            l10n (dict, optional): Dictionary to translate tags to strings. Defaults to None.
+            devclass (str, optional): The device class of the device used to extract the value.
+                Defaults to "1".
+
+        Returns:
+            bool: Extracting value successful
         """
         try:
             res = result_body[self.key]
@@ -107,50 +108,79 @@ class Sensor:  # pylint: disable=too-many-instance-attributes
 class Sensors:
     """SMA Sensors."""
 
-    def __init__(self, sensors=None):
+    def __init__(self, sensors: Union[Sensor, List[Sensor], None] = None):
         """Init Sensors.
 
         Args:
-            sensors: One or a list of sensors to add on init
+            sensors (Union[Sensor, List[Sensor], None], optional): One or a list of sensors
+                to add on init. Defaults to None.
         """
-        self.__s = []
+        self.__s: List[Sensor] = []
 
         if sensors:
             self.add(sensors)
 
-    def __len__(self):
-        """Length."""
+    def __len__(self) -> int:
+        """Length of Sensor list.
+
+        Returns:
+            int: Number of Sensor objects
+        """
         return len(self.__s)
 
-    # pylint: disable=R1710
-    def __contains__(self, key):
-        """Check if a sensor is defined."""
+    def __contains__(self, key: str) -> bool:
+        """Check if a sensor is defined.
+
+        Args:
+            key (str): [description]
+
+        Returns:
+            bool: [description]
+        """
         try:
             if self[key]:
                 return True
         except KeyError:
-            return False
+            pass
+        return False
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> Sensor:
         """Get a sensor.
 
         Args:
-            key: Either the name or key of the Sensor
+            key (str): Either the name or key of the Sensor
+
+        Raises:
+            KeyError: Item was not found
+
+        Returns:
+            Sensor: The matching Sensor object
         """
         for sen in self.__s:
             if key in (sen.name, sen.key):
                 return sen
         raise KeyError(key)
 
-    def __iter__(self):
-        """Iterate Sensor objects."""
+    def __iter__(self) -> Iterator[Sensor]:
+        """Iterate Sensor objects.
+
+        Yields:
+            Iterator[Sensor]: Sensor iterator
+        """
         return self.__s.__iter__()
 
-    def add(self, sensor):
-        """Add a sensor, warning if it exists.
+    def add(self, sensor: Union[Sensor, List[Sensor]]) -> None:
+        """Add a sensor, logs warning if it exists.
+
+        A copy of sensor, or the sensors in the list, will be added.
+        If the sensor name already exists it will be overwritten.
+        If the sensor key already exists it will not be added.
 
         Args:
-            sensor: One or a list of sensors to add
+            sensor (Sensor, List[Sensor]): One or a list of sensors to add
+
+        Raises:
+            TypeError: Argument sensor does not match an expected type
         """
         if isinstance(sensor, (list, tuple)):
             for sss in sensor:
